@@ -151,10 +151,10 @@ class install_install extends module
 			'LEGEND_EXPLAIN'	=> $lang['PHP_SETTINGS_EXPLAIN'],
 		));
 
-		// Test the minimum PHP version
+		// Test the minimum and maximum version of PHP
 		$php_version = PHP_VERSION;
 
-		if (version_compare($php_version, '5.3.3') < 0)
+		if ((version_compare($php_version, '5.3.3') < 0) || (version_compare($php_version, '7.0.0-dev', '>=')))
 		{
 			$result = '<strong style="color:red">' . $lang['NO'] . '</strong>';
 		}
@@ -404,7 +404,7 @@ class install_install extends module
 			));
 		}
 
-		// Can we find Imagemagick anywhere on the system?
+		// Can we find ImageMagick anywhere on the system?
 		$exe = (DIRECTORY_SEPARATOR == '\\') ? '.exe' : '';
 
 		$magic_home = getenv('MAGICK_HOME');
@@ -1459,7 +1459,7 @@ class install_install extends module
 	*/
 	function build_search_index($mode, $sub)
 	{
-		global $db, $lang, $phpbb_root_path, $phpEx, $config, $auth, $user;
+		global $db, $lang, $phpbb_root_path, $phpbb_dispatcher, $phpEx, $config, $auth, $user;
 
 		// Obtain any submitted data
 		$data = $this->get_submitted_data();
@@ -1492,7 +1492,7 @@ class install_install extends module
 		set_config_count(null, null, null, $config);
 
 		$error = false;
-		$search = new \phpbb\search\fulltext_native($error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user);
+		$search = new \phpbb\search\fulltext_native($error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user, $phpbb_dispatcher);
 
 		$sql = 'SELECT post_id, post_subject, post_text, poster_id, forum_id
 			FROM ' . POSTS_TABLE;
