@@ -42,7 +42,7 @@ class file_downloader
 		$this->error_number = 0;
 		$this->error_string = '';
 
-		if ($socket = @fsockopen($host, $port, $this->error_number, $this->error_string, $timeout))
+		if ($socket = @fsockopen(($port == 443 ? 'tls://' : '') . $host, $port, $this->error_number, $this->error_string, $timeout))
 		{
 			@fputs($socket, "GET $directory/$filename HTTP/1.0\r\n");
 			@fputs($socket, "HOST: $host\r\n");
@@ -91,7 +91,7 @@ class file_downloader
 			}
 			else
 			{
-				throw new \RuntimeException('FSOCK_DISABLED');
+				throw new \phpbb\exception\runtime_exception('FSOCK_DISABLED');
 			}
 		}
 
