@@ -126,6 +126,26 @@ gen_sort_selects($limit_days, $sort_by_text, $sort_days, $sort_key, $sort_dir, $
 $phpbb_content_visibility = $phpbb_container->get('content.visibility');
 $pagination = $phpbb_container->get('pagination');
 
+/**
+* This event allows you to alter the above parameters, such as keywords and submit
+*
+* @event core.search_modify_submit_parameters
+* @var	string	keywords	The search keywords
+* @var	string	author		Specifies the author match, when ANONYMOUS is also a search-match
+* @var	int		author_id	ID of the author to search by
+* @var	string	search_id	Predefined search type name
+* @var	bool	submit		Whether or not the form has been submitted
+* @since 3.1.10-RC1
+*/
+$vars = array(
+	'keywords',
+	'author',
+	'author_id',
+	'search_id',
+	'submit',
+);
+extract($phpbb_dispatcher->trigger_event('core.search_modify_submit_parameters', compact($vars)));
+
 if ($keywords || $author || $author_id || $search_id || $submit)
 {
 	// clear arrays
@@ -274,7 +294,7 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 	}
 	// We do some additional checks in the module to ensure it can actually be utilised
 	$error = false;
-	$search = new $search_type($error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user);
+	$search = new $search_type($error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user, $phpbb_dispatcher);
 
 	if ($error)
 	{
@@ -320,7 +340,10 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 	* @var	array	ex_fid_ary		Array of excluded forum ids
 	* @var	array	author_id_ary	Array of exclusive author ids
 	* @var	string	search_id		The id of the search request
+	* @var	array	id_ary			Array of post or topic ids for search result
+	* @var	string	show_results	'posts' or 'topics' type of ids
 	* @since 3.1.3-RC1
+	* @changed 3.1.10-RC1 Added id_ary, show_results
 	*/
 	$vars = array(
 		'keywords',
@@ -328,6 +351,8 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 		'ex_fid_ary',
 		'author_id_ary',
 		'search_id',
+		'id_ary',
+		'show_results',
 	);
 	extract($phpbb_dispatcher->trigger_event('core.search_modify_param_before', compact($vars)));
 
@@ -427,6 +452,8 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 
 				gen_sort_selects($limit_days, $sort_by_text, $sort_days, $sort_key, $sort_dir, $s_limit_days, $s_sort_key, $s_sort_dir, $u_sort_param);
 				$s_sort_key = $s_sort_dir = $u_sort_param = $s_limit_days = '';
+
+				$template->assign_var('U_MARK_ALL_READ', ($user->data['is_registered'] || $config['load_anon_lastread']) ? append_sid("{$phpbb_root_path}index.$phpEx", 'hash=' . generate_link_hash('global') . '&amp;mark=forums&amp;mark_time=' . time()) : '');
 			break;
 
 			case 'newposts':
@@ -483,6 +510,24 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 		}
 	}
 
+	/**
+	* Event to modify data after pre-made searches
+	*
+	* @event core.search_modify_param_after
+	* @var	string	l_search_title	The title of the search page
+	* @var	string	search_id		Predefined search type name
+	* @var	string	show_results	Display topics or posts
+	* @var	string	sql				SQL query corresponding to the pre-made search id
+	* @since 3.1.7-RC1
+	*/
+	$vars = array(
+		'l_search_title',
+		'search_id',
+		'show_results',
+		'sql',
+	);
+	extract($phpbb_dispatcher->trigger_event('core.search_modify_param_after', compact($vars)));
+
 	// show_results should not change after this
 	$per_page = ($show_results == 'posts') ? $config['posts_per_page'] : $config['topics_per_page'];
 	$total_match_count = 0;
@@ -550,6 +595,48 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 		$total_match_count = $search->author_search($show_results, $firstpost_only, $sort_by_sql, $sort_key, $sort_dir, $sort_days, $ex_fid_ary, $m_approve_posts_fid_sql, $topic_id, $author_id_ary, $sql_author_match, $id_ary, $start, $per_page);
 	}
 
+	/**
+	* Event to search otherwise than by keywords or author
+	*
+	* @event core.search_backend_search_after
+	* @var	string		show_results				'posts' or 'topics' type of ids
+	* @var	string		search_fields				The data fields to search in
+	* @var	string		search_terms				Is either 'all' (use query as entered, words without prefix should default to "have to be in field") or 'any' (ignore search query parts and just return all posts that contain any of the specified words)
+	* @var	array		sort_by_sql					Array of SQL sorting instructions
+	* @var	string		sort_key					The sort key
+	* @var	string		sort_dir					The sort direction
+	* @var	int			sort_days					Limit the age of results
+	* @var	array		ex_fid_ary					Array of excluded forum ids
+	* @var	string		m_approve_posts_fid_sql		Specifies which types of posts the user can view in which forums
+	* @var	int			topic_id					is set to 0 or a topic id, if it is not 0 then only posts in this topic should be searched
+	* @var	array		author_id_ary				Array of exclusive author ids
+	* @var	string		sql_author_match			Specifies the author match, when ANONYMOUS is also a search-match
+	* @var	array		id_ary						Array of post or topic ids for search result
+	* @var	int			start						The starting id of the results
+	* @var	int			per_page					Number of ids each page is supposed to contain
+	* @var	int			total_match_count			The total number of search matches
+	* @since 3.1.10-RC1
+	*/
+	$vars = array(
+		'show_results',
+		'search_fields',
+		'search_terms',
+		'sort_by_sql',
+		'sort_key',
+		'sort_dir',
+		'sort_days',
+		'ex_fid_ary',
+		'm_approve_posts_fid_sql',
+		'topic_id',
+		'author_id_ary',
+		'sql_author_match',
+		'id_ary',
+		'start',
+		'per_page',
+		'total_match_count',
+	);
+	extract($phpbb_dispatcher->trigger_event('core.search_backend_search_after', compact($vars)));
+
 	$sql_where = '';
 
 	if (sizeof($id_ary))
@@ -594,6 +681,27 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 	$u_search .= ($search_fields != 'all') ? '&amp;sf=' . $search_fields : '';
 	$u_search .= ($return_chars != 300) ? '&amp;ch=' . $return_chars : '';
 
+	/**
+	* Event to add or modify search URL parameters
+	*
+	* @event core.search_modify_url_parameters
+	* @var	string	u_search		Search URL parameters string
+	* @var	string	search_id		Predefined search type name
+	* @var	string	show_results	String indicating the show results mode
+	* @var	string	sql_where		The SQL WHERE string used by search to get topic data
+	* @var	int		total_match_count	The total number of search matches
+	* @since 3.1.7-RC1
+	* @changed 3.1.10-RC1 Added show_results, sql_where, total_match_count
+	*/
+	$vars = array(
+		'u_search',
+		'search_id',
+		'show_results',
+		'sql_where',
+		'total_match_count',
+	);
+	extract($phpbb_dispatcher->trigger_event('core.search_modify_url_parameters', compact($vars)));
+
 	if ($sql_where)
 	{
 		if ($show_results == 'posts')
@@ -704,6 +812,8 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 				$tracking_topics = ($tracking_topics) ? tracking_unserialize($tracking_topics) : array();
 			}
 
+			$sql_order_by = $sort_by_sql[$sort_key] . ' ' . (($sort_dir == 'd') ? 'DESC' : 'ASC');
+
 			/**
 			* Event to modify the SQL query before the topic data is retrieved
 			*
@@ -712,16 +822,30 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 			* @var	string	sql_from		The SQL FROM string used by search to get topic data
 			* @var	string	sql_where		The SQL WHERE string used by search to get topic data
 			* @var	int		total_match_count	The total number of search matches
+			* @var	array	sort_by_sql		Array of SQL sorting instructions
+			* @var	string	sort_dir		The sorting direction
+			* @var	string	sort_key		The sorting key
+			* @var	string	sql_order_by	The SQL ORDER BY string used by search to get topic data
 			* @since 3.1.0-a1
 			* @changed 3.1.0-RC5 Added total_match_count
+			* @changed 3.1.7-RC1 Added sort_by_sql, sort_dir, sort_key, sql_order_by
 			*/
-			$vars = array('sql_select', 'sql_from', 'sql_where', 'total_match_count');
+			$vars = array(
+				'sql_select',
+				'sql_from',
+				'sql_where',
+				'total_match_count',
+				'sort_by_sql',
+				'sort_dir',
+				'sort_key',
+				'sql_order_by',
+			);
 			extract($phpbb_dispatcher->trigger_event('core.search_get_topic_data', compact($vars)));
 
 			$sql = "SELECT $sql_select
 				FROM $sql_from
-				WHERE $sql_where";
-			$sql .= ' ORDER BY ' . $sort_by_sql[$sort_key] . ' ' . (($sort_dir == 'd') ? 'DESC' : 'ASC');
+				WHERE $sql_where
+				ORDER BY $sql_order_by";
 		}
 		$result = $db->sql_query($sql);
 		$result_topic_id = 0;
@@ -1167,14 +1291,25 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 	* Modify the title and/or load data for the search results page
 	*
 	* @event core.search_results_modify_search_title
-	* @var	int		author_id		ID of the author to search by
-	* @var	string	l_search_title	The title of the search page
-	* @var	string	search_id		Predefined search type name
-	* @var	string	show_results	Search results output mode - topics or posts
-	* @var	int		start			The starting id of the results
+	* @var	int		author_id			ID of the author to search by
+	* @var	string	l_search_title		The title of the search page
+	* @var	string	search_id			Predefined search type name
+	* @var	string	show_results		Search results output mode - topics or posts
+	* @var	int		start				The starting id of the results
+	* @var	int		total_match_count	The count of search results
+	* @var	string	keywords			The search keywords
 	* @since 3.1.0-RC4
+	* @changed 3.1.6-RC1 Added total_match_count and keywords
 	*/
-	$vars = array('author_id', 'l_search_title', 'search_id', 'show_results', 'start');
+	$vars = array(
+		'author_id',
+		'l_search_title',
+		'search_id',
+		'show_results',
+		'start',
+		'total_match_count',
+		'keywords',
+	);
 	extract($phpbb_dispatcher->trigger_event('core.search_results_modify_search_title', compact($vars)));
 
 	page_header(($l_search_title) ? $l_search_title : $user->lang['SEARCH']);
@@ -1188,6 +1323,7 @@ if ($keywords || $author || $author_id || $search_id || $submit)
 }
 
 // Search forum
+$rowset = array();
 $s_forums = '';
 $sql = 'SELECT f.forum_id, f.forum_name, f.parent_id, f.forum_type, f.left_id, f.right_id, f.forum_password, f.enable_indexing, fa.user_id
 	FROM ' . FORUMS_TABLE . ' f
@@ -1196,11 +1332,27 @@ $sql = 'SELECT f.forum_id, f.forum_name, f.parent_id, f.forum_type, f.left_id, f
 	ORDER BY f.left_id ASC";
 $result = $db->sql_query($sql);
 
+while ($row = $db->sql_fetchrow($result))
+{
+	$rowset[(int) $row['forum_id']] = $row;
+}
+$db->sql_freeresult($result);
+
 $right = $cat_right = $padding_inc = 0;
 $padding = $forum_list = $holding = '';
 $pad_store = array('0' => '');
 
-while ($row = $db->sql_fetchrow($result))
+/**
+* Modify the forum select list for advanced search page
+*
+* @event core.search_modify_forum_select_list
+* @var	array	rowset	Array with the forums list data
+* @since 3.1.10-RC1
+*/
+$vars = array('rowset');
+extract($phpbb_dispatcher->trigger_event('core.search_modify_forum_select_list', compact($vars)));
+
+foreach ($rowset as $row)
 {
 	if ($row['forum_type'] == FORUM_CAT && ($row['left_id'] + 1 == $row['right_id']))
 	{
@@ -1272,8 +1424,8 @@ if ($holding)
 	$s_forums .= $holding;
 }
 
-$db->sql_freeresult($result);
 unset($pad_store);
+unset($rowset);
 
 if (!$s_forums)
 {
